@@ -6,7 +6,12 @@
 -- complain if script is sourced in psql, rather than via CREATE EXTENSION
 \echo Use "CREATE EXTENSION hypopg" to load this file. \quit
 
-SET client_encoding = 'UTF8';
+SET LOCAL client_encoding = 'UTF8';
+
+CREATE FUNCTION hypopg_reset_index()
+    RETURNS void
+    LANGUAGE C VOLATILE COST 100
+AS '$libdir/hypopg', 'hypopg_reset_index';
 
 CREATE FUNCTION hypopg_reset()
     RETURNS void
