@@ -332,8 +332,61 @@ hypo_index_store_parsetree(IndexStmt *node, const char *queryString)
 	ListCell   *lc;
 	int			attn;
 
-	relid =
-		RangeVarGetRelid(node->relation, AccessShareLock, false);
+	/*
+	 * Support for hypothetical BRIN indexes is broken in some minor versions
+	 * of pg10, pg11 and pg12.  For simplicity, check PG_VERSION_NUM rather
+	 * than the real instance version, which should should be right most of the
+	 * time.  When it's not, the only effect is to have a less user-friendly
+	 * error message.
+	 */
+#if ((PG_VERSION_NUM >= 100000 && PG_VERSION_NUM < 100012) || \
+	(PG_VERSION_NUM >= 110000 && PG_VERSION_NUM < 110007) || \
+	(PG_VERSION_NUM >= 120000 && PG_VERSION_NUM < 120002))
+	if (get_am_oid(node->accessMethod, true) == BRIN_AM_OID)
+	{
+		elog(ERROR, "hypopg: BRIN hypothetical indexes are only supported"
+				" with PostgreSQL "
+#if PG_VERSION_NUM >= 120000
+				"12.2"
+#else
+#if PG_VERSION_NUM >= 110000
+				"11.7"
+#else
+				"10.12"
+#endif					/* pg 11 */
+#endif					/* pg 12 */
+				" and later.");
+	}
+#endif
+
+	relid = RangeVarGetRelid(node->relation, AccessShareLock, false);
+
+	/* Some sanity checks */
+	switch (get_rel_relkind(relid))
+	{
+#if PG_VERSION_NUM >= 90300
+		case RELKIND_MATVIEW:
+#endif
+#if PG_VERSION_NUM >= 110000
+		case RELKIND_PARTITIONED_TABLE:
+#endif
+		case RELKIND_RELATION:
+			/* this is supported */
+			break;
+#if PG_VERSION_NUM >= 100000 && PG_VERSION_NUM < 110000
+		case RELKIND_PARTITIONED_TABLE:
+			elog(ERROR, "hypopg: cannot create hypothetical index on"
+				 " partitioned table \"%s\"", node->relation->relname);
+			break;
+#endif
+		default:
+			elog(ERROR, "hypopg: \"%s\" is not a table"
+#if PG_VERSION_NUM >= 90300
+				 " or materialized view"
+#endif
+				 ,
+				 node->relation->relname);
+	}
 
 	/* Run parse analysis ... */
 	node = transformIndexStmt(relid, node, queryString);
@@ -1319,7 +1372,7 @@ hypopg_relation_size(PG_FUNCTION_ARGS)
 		}
 	}
 
-	PG_RETURN_INT64(pages * BLCKSZ);
+	PG_RETURN_INT64(pages * 1.0L * BLCKSZ);
 }
 
 /*
@@ -1396,7 +1449,7 @@ hypopg_get_indexdef(PG_FUNCTION_ARGS)
 			if (indexpr_item == NULL)
 				elog(ERROR, "too few entries in indexprs list");
 			indexkey = (Node *) lfirst(indexpr_item);
-			indexpr_item = lnext(indexpr_item);
+			indexpr_item = lnext(entry->indexprs, indexpr_item);
 
 			/* Deparse */
 			str = deparse_expression(indexkey, context, false, false);
@@ -1525,7 +1578,7 @@ hypo_set_indexname(hypoIndex * entry, char *indexname)
 		totalsize = NAMEDATALEN;
 
 	/* eventually truncate the given indexname at NAMEDATALEN-1 if needed */
-	strncpy(entry->indexname, oid, strlen(oid));
+	strcpy(entry->indexname, oid);
 	strncat(entry->indexname, indexname, totalsize - strlen(oid) - 1);
 }
 
@@ -1546,7 +1599,7 @@ hypo_estimate_index_simple(hypoIndex * entry, BlockNumber *pages, double *tuples
 	rel = makeNode(RelOptInfo);
 
 	/* Open the hypo index' relation */
-	relation = heap_open(entry->relid, AccessShareLock);
+	relation = table_open(entry->relid, AccessShareLock);
 
 	if (!RelationNeedsWAL(relation) && RecoveryInProgress())
 		ereport(ERROR,
@@ -1567,7 +1620,7 @@ hypo_estimate_index_simple(hypoIndex * entry, BlockNumber *pages, double *tuples
 					  &rel->pages, &rel->tuples, &rel->allvisfrac);
 
 	/* Close the relation and release the lock now */
-	heap_close(relation, AccessShareLock);
+	table_close(relation, AccessShareLock);
 
 	hypo_estimate_index(entry, rel);
 	*pages = entry->pages;
